% Copyright (c) 2012, Jan Philipp Kruse and Harro Schmeling, Universitt Frankfurt
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without modification, 
% are permitted provided that the following conditions are met:
% 
% Redistributions of source code must retain the above copyright notice, 
% this list of conditions and the following disclaimer.
% Redistributions in binary form must reproduce the above copyright notice, 
% this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
% IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
% OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
% EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

function calculateMmodelforgui(namefield,modelname,K0,mu0,Kf,rho0,drho,corrf,betamin,betamax,betanum,alpha1,kappa,alpha3,c,nmax,unit,datatable)

global Mmodels;

missmax1    = 10^-11*K0;
missmax2    = 10^-11*mu0;
beta    = logspace(log10(betamin),log10(betamax),betanum);

disp(['model Name = ',modelname])
    
Ku      = zeros(1,length(beta));
muu     = zeros(1,length(beta));
Au      = zeros(1,length(beta));
nuu     = zeros(1,length(beta));

Kbar    = zeros(1,length(beta));
Abar    = zeros(1,length(beta));
nubar   = zeros(1,length(beta));

Kiso    = zeros(1,length(beta));
muisoo  = zeros(1,length(beta));
Aiso    = zeros(1,length(beta));
nuiso   = zeros(1,length(beta));

Kr      = zeros(1,length(beta));
mur     = zeros(1,length(beta));
nur     = zeros(1,length(beta));

vpu     = zeros(1,length(beta));
vsu     = zeros(1,length(beta));
vpvsu  = zeros(1,length(beta));

vpr     = zeros(1,length(beta));
vsr     = zeros(1,length(beta));
vpvsr  = zeros(1,length(beta));

rho     = zeros(1,length(beta));

phi_tmp = phi(alpha3);
g_tmp   = g(alpha3,phi_tmp);

wait    = waitbar(0,'Calculating ...');
set(wait,'Name','Calculating ...');

for i=1:length(beta)
    
    rho(i)              = rho0*(1-drho/rho0*beta(i));
    
    %% -------------------------------- %%
    %% UNRELAXED MODULI_GUI             %%
    %% -------------------------------- %%
    
    %% START INDEX                      %%
    n                   = 2;
    
    %% SET INITIAL CONDITIONS           %%
    %% -------------------------------- %%
    
    %% K                                %%
    Ku_tmp(1)           = K0;
    Ku_tmp(2)           = 0;
    
    %% MU                               %%
    muu_tmp(1)          = mu0;
    
    %% INITIAL ERROR                    %%
    miss1               = K0;
    miss2               = mu0;
    
    %% -------------------------------- %%
    %% RECURSION FOR KU & MUU           %%
    %% -------------------------------- %%
    
    while (((miss1>=missmax1) || (miss2>=missmax2)) && (n<=nmax))
        K_tmp           = 0;
        mu_tmp          = 0;
        for j=1:3
            nu_tmp      = poisson(Ku_tmp(n-1),muu_tmp(n-1));
            R_tmp       = R(Ku_tmp(n-1),muu_tmp(n-1));
            B_tmp       = B(j,Kf,Ku_tmp(n-1),nu_tmp,kappa,muu_tmp(n-1));
            theta_tmp   = theta(j,Ku_tmp(n-1),muu_tmp(n-1),nu_tmp,alpha1,kappa,g_tmp,phi_tmp,R_tmp);
            D_tmp       = D(j,Kf,Ku_tmp(1),theta_tmp);
            A_tmp       = A(j,alpha1,alpha3,Ku_tmp(n-1),g_tmp,R_tmp,D_tmp,phi_tmp,B_tmp,muu_tmp(n-1),nu_tmp,kappa,Kf);
            mu_tmp      = mu_tmp+muunrelaxed(A_tmp,c(j,1)*beta(i),c(j,2)*beta(i));
            K_tmp       = K_tmp+KModulunrelaxed(Kf,K0,c(j,1)*beta(i),c(j,2)*beta(i),theta_tmp,Ku_tmp(n-1));
        end
        Ku_tmp(n)       = 1/(1/K0+K_tmp);
        muu_tmp(n)      = 1/(1/mu0+mu_tmp);
        
        %% ERROR FOR MY AND K               %%
        miss1           = abs(Ku_tmp(n)-Ku_tmp(n-1));
        miss2           = abs(muu_tmp(n)-muu_tmp(n-1));
        %% -------------------------------- %%
        
        n               = n+1;
        
    end
    %% -------------------------------- %%
    
    % disp(['K_u calculated for beta = ',num2str(beta(i)),' ! -> error @ iteration ',num2str(n-1),' = ',num2str(miss1)]);
    
    %% SET VALUES FOR UNRELAXED TERMS   %%
    %% -------------------------------- %%
    Ku(i)               = Ku_tmp(n-1);
    muu(i)              = muu_tmp(n-1);
    Au(i)               = A_tmp;
    nuu(i)              = nu_tmp;
    vpu(i)              = sqrt((Ku(i)*corrf+4/3*muu(i)*corrf)/rho(i))*10^-3;
    vsu(i)              = sqrt((muu(i)*corrf)/rho(i))*10^-3;
    vpvsu(i)            = vpu(i)/vsu(i);
    %% -------------------------------- %%
    
    
    %% KMODULI' and RELAXED SHEARMODULI %%
    %% -------------------------------- %%
    
    %% START INDEX                      %%
    n                   = 2;
    %% SET INITIAL CONDITIONS           %%
    %% -------------------------------- %%
    
    %% K'                               %%
    Kbar_tmp(1)         = K0;
    Kbar_tmp(2)         = 0;
    
    %% MY_R                             %%
    mubar_tmp(1)        = mu0;
    
    %% INITIAL ERROR                    %%
    miss1               = 1;
    miss2               = 1;
    
    %% -------------------------------- %%
    %% RECURSION FOR K' & MY_R          %%
    %% -------------------------------- %%
    
    while (((miss1>=missmax1) || (miss2>=missmax2)) && (n<=nmax))
        K_tmp           = 0;
        mu_tmp          = 0;
        for j=1:3
            nu_tmp      = poisson(Kbar_tmp(n-1),mubar_tmp(n-1));
            R_tmp       = R(Kbar_tmp(n-1),mubar_tmp(n-1));
            B1_tmp      = B(j,Kf,Kbar_tmp(n-1),nu_tmp,kappa,mubar_tmp(n-1));
            B2_tmp      = B(j,0,Kbar_tmp(n-1),nu_tmp,kappa,mubar_tmp(n-1));
            theta_tmp   = theta(j,Kbar_tmp(n-1),mubar_tmp(n-1),nu_tmp,alpha1,kappa,g_tmp,phi_tmp,R_tmp);
            D1_tmp      = D(j,Kf,Kbar_tmp(1),theta_tmp);
            D2_tmp      = D(j,0,Kbar_tmp(1),theta_tmp);
            A1_tmp      = A(j,alpha1,alpha3,Kbar_tmp(n-1),g_tmp,R_tmp,D1_tmp,phi_tmp,B1_tmp,mubar_tmp(n-1),nu_tmp,kappa,Kf);
            A2_tmp      = A(j,alpha1,alpha3,Kbar_tmp(n-1),g_tmp,R_tmp,D2_tmp,phi_tmp,B2_tmp,mubar_tmp(n-1),nu_tmp,kappa,0);
            mu_tmp      = mu_tmp+murelaxed(A1_tmp,A2_tmp,c(j,1)*beta(i),c(j,2)*beta(i));
            K_tmp       = K_tmp+KModulbar(Kf,K0,theta_tmp,Kbar_tmp(n-1),c(j,1)*beta(i),c(j,2)*beta(i));
        end
        
        Kbar_tmp(n)     = 1/(1/K0+K_tmp);
        mubar_tmp(n)    = 1/(1/mu0+mu_tmp);
        
        %% ERROR FOR MY_R AND K'            %%
        miss1           = abs(Kbar_tmp(n)-Kbar_tmp(n-1));
        miss2           = abs(mubar_tmp(n)-mubar_tmp(n-1));
        %% -------------------------------- %%
        
        n               = n+1;
    end
    %% -------------------------------- %%
    
    % disp(['K'' calculated for beta = ',num2str(beta(i)),' ! -> error @ iteration ',num2str(n-1),' = ',num2str(miss1)]);
    
    %% SET VALUES FOR BAR AND MY_RELAXED TERMS %%
    %% --------------------------------------- %%
    mur(i)              = mubar_tmp(n-1);
    Kbar(i)             = Kbar_tmp(n-1);
    Abar(i)             = A_tmp;
    nubar(i)            = nu_tmp;
    %% --------------------------------------- %%
    
    %% KMODULI ISOLATED                        %%
    %% --------------------------------------- %%
    
    %% START INDEX                             %%
    n                   = 2;
    
    %% SET INITIAL CONDITIONS                  %%
    %% --------------------------------------- %%
    
    %% KISO                                    %%
    Kiso_tmp(1)         = K0;
    Kiso_tmp(2)         = 0;
    
    %% MYISO                                   %%
    muiso_tmp(1)        = mu0;
    
    %% INITIAL ERROR                           %%
    miss1               = 1;
    miss2               = 1;
    %% --------------------------------------- %%
    
    %% RECURSION FOR KISO & MUISO              %%
    %% --------------------------------------- %%
    while (((miss1>=missmax1) || (miss2>=missmax2)) && (n<=nmax))
        K_tmp           = 0;
        mu_tmp          = 0;
        for j=1:3
            betamod     = (c(j,1)*beta(i))/(1-c(j,2)*beta(i));
            nu_tmp      = poisson(Kiso_tmp(n-1),muiso_tmp(n-1));
            R_tmp       = R(Kiso_tmp(n-1),muiso_tmp(n-1));
            B_tmp       = B(j,Kf,Kiso_tmp(n-1),nu_tmp,kappa,muiso_tmp(n-1));
            theta_tmp   = theta(j,Kiso_tmp(n-1),muiso_tmp(n-1),nu_tmp,alpha1,kappa,g_tmp,phi_tmp,R_tmp);
            D_tmp       = D(j,Kf,Kiso_tmp(1),theta_tmp);
            A_tmp       = A(j,alpha1,alpha3,Kiso_tmp(n-1),g_tmp,R_tmp,D_tmp,phi_tmp,B_tmp,muiso_tmp(n-1),nu_tmp,kappa,Kf);
            mu_tmp      = mu_tmp+muiso(A_tmp,betamod);
            K_tmp       = K_tmp+KModuliso(Kf,K0,betamod,theta_tmp,Kiso_tmp(n-1));
        end
        
        Kiso_tmp(n)     = 1/(1/K0+K_tmp);
        muiso_tmp(n)    = 1/(1/mu0+mu_tmp);
        
        %% ERROR FOR MY_ISO AND K_ISO              %%
        miss1           = abs(Kiso_tmp(n)-Kiso_tmp(n-1));
        miss2           = abs(muiso_tmp(n)-muiso_tmp(n-1));
        %% --------------------------------------- %%
        
        n               = n+1;
    end
    %% --------------------------------------- %%
    
    % disp(['K_iso calculated for betamod = ',num2str(beta(i)),' ! -> error @ iteration ',num2str(n-1),' = ',num2str(miss1)]);
    
    %% SET VALUES FOR ISOLATED TERMS           %%
    %% --------------------------------------- %%
    Kiso(i)             = Kiso_tmp(n-1);
    muisoo(i)           = muiso_tmp(n-1);
    Aiso(i)             = A_tmp;
    nuiso(i)            = nu_tmp;
    %% --------------------------------------- %%
    
    %% SET VALUES FOR RELAXED TERMS            %%
    %% --------------------------------------- %%
    Kr(i)               = KModulrelaxed(Kf,Kiso(i),Kbar(i),(c(1,2)+c(2,2)+c(3,2))*beta(i));
    nur(i)              = poisson(Kr(i),mur(i));
    vpr(i)              = sqrt((Kr(i)*corrf+4/3*mur(i)*corrf)/rho(i))*10^-3;
    vsr(i)              = sqrt((mur(i)*corrf)/rho(i))*10^-3;
    vpvsr(i)            = vpr(i)/vsr(i);
    %% --------------------------------------- %%
    
    waitbar(i/length(beta),wait,['beta=',sprintf('%1.6f',beta(i))]);
end
close(wait);

tablemodel      = datatable.getModel();
introw          = tablemodel.getRowCount();
tablemodel.setRowCount(introw+1);

datatable.setValueAt(modelname,introw,0);
datatable.setValueAt(betamin,introw,1);
datatable.setValueAt(betamax,introw,2);
datatable.setValueAt(K0,introw,3);
datatable.setValueAt(Kf,introw,4);
datatable.setValueAt(mu0,introw,5);
datatable.setValueAt(rho0,introw,6);
datatable.setValueAt(drho,introw,7);
datatable.setValueAt(alpha1,introw,8);
datatable.setValueAt(kappa,introw,9);
datatable.setValueAt(alpha3,introw,10);
datatable.setValueAt(c(1,1),introw,11);
datatable.setValueAt(c(1,2),introw,12);
datatable.setValueAt(c(2,1),introw,13);
datatable.setValueAt(c(2,2),introw,14);
datatable.setValueAt(c(3,1),introw,15);
datatable.setValueAt(c(3,2),introw,16);
datatable.setValueAt(nmax,introw,17);
datatable.setValueAt(unit,introw,18);

datatable.setModel(tablemodel);

mod_ind                 = tablemodel.getRowCount();

Mmodels(mod_ind).id         = tablemodel.getRowCount();
Mmodels(mod_ind).name       = modelname;
Mmodels(mod_ind).betamin    = betamin;
Mmodels(mod_ind).betamax    = betamax;
Mmodels(mod_ind).beta       = beta;
Mmodels(mod_ind).K_0        = K0;
Mmodels(mod_ind).mu_0       = mu0;
Mmodels(mod_ind).K_f        = Kf;
Mmodels(mod_ind).rho_0      = rho0;
Mmodels(mod_ind).drho       = drho;
Mmodels(mod_ind).alpha1     = alpha1;
Mmodels(mod_ind).alpha3     = alpha3;
Mmodels(mod_ind).kappa      = kappa;
Mmodels(mod_ind).c1i        = c(1,1);
Mmodels(mod_ind).c1c        = c(1,2);
Mmodels(mod_ind).c2i        = c(2,1);
Mmodels(mod_ind).c2c        = c(2,2);
Mmodels(mod_ind).c3i        = c(3,1);
Mmodels(mod_ind).c3c        = c(3,2);
Mmodels(mod_ind).nmax       = nmax;
Mmodels(mod_ind).K_u        = Ku;
Mmodels(mod_ind).K_bar      = Kbar;
Mmodels(mod_ind).K_iso      = Kiso;
Mmodels(mod_ind).K_r        = Kr;
Mmodels(mod_ind).mu_u       = muu;
Mmodels(mod_ind).mu_iso     = muisoo;
Mmodels(mod_ind).mu_r       = mur;
Mmodels(mod_ind).nu_u       = nuu;
Mmodels(mod_ind).nu_iso     = nuiso;
Mmodels(mod_ind).nu_bar     = nubar;
Mmodels(mod_ind).nu_r       = nur;
Mmodels(mod_ind).A_u        = Au;
Mmodels(mod_ind).A_bar      = Abar;
Mmodels(mod_ind).A_iso      = Aiso;
Mmodels(mod_ind).rho        = rho;
Mmodels(mod_ind).v_pu       = vpu;
Mmodels(mod_ind).v_su       = vsu;
Mmodels(mod_ind).ratio_psu  = vpvsu;
Mmodels(mod_ind).v_pr       = vpr;
Mmodels(mod_ind).v_sr       = vsr;
Mmodels(mod_ind).ratio_psr  = vpvsr;
Mmodels(mod_ind).unit       = unit;

namefield.setText(['model ',num2str(tablemodel.getRowCount()+1)]);

end
