% Copyright (c) 2012, Jan Philipp Kruse and Harro Schmeling, Universitt Frankfurt
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without modification, 
% are permitted provided that the following conditions are met:
% 
% Redistributions of source code must retain the above copyright notice, 
% this list of conditions and the following disclaimer.
% Redistributions in binary form must reproduce the above copyright notice, 
% this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
% IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
% OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
% EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

function calculateVmodelforgui(namefield,modelname,eta_s0,betamin,betamax,betanum,alpha1,kappa,alpha3,c,nmax,unit,datatable)

global Vmodels;

missmax     = 10^-11*eta_s0;
beta        = logspace(log10(betamin),log10(betamax),betanum);

disp(['model Name = ',modelname])

A_r         = zeros(1,length(beta));
eta_b       = zeros(1,length(beta));
eta_s       = zeros(1,length(beta));
nu_r        = zeros(1,length(beta));

phi_tmp     = phi(alpha3);
g_tmp       = g(alpha3,phi_tmp);

wait        = waitbar(0,'Calculating ...');
set(wait,'Name','Calculating ...');

for i=1:length(beta)
    
    %% ETA' and RELAXED SHEARVISCOSITY %%
    %% -------------------------------- %%
    
    %% START INDEX %%
    n = 2;
    
    %% SET INITIAL CONDITIONS %%
    %% ---------------------- %%
    %% ETA_BR %%
    eta_btmp1(1) = eta_s0; % c1*eta_s0*(c2(u)-beta(1))/beta(1);
    eta_btmp1(2) = 0;
    %% ------ %%
    
    %% ETA_SR %%
    eta_stmp1(1) = eta_s0;
    %% ------ %%
    
    %% INITIAL ERROR %%
    miss1 = eta_s0;
    miss2 = eta_s0;
    %% ------------- %%
    
    %% RECURSION FOR ETA_BR & ETA_SR %%
    %% ----------------------------- %%
    while (((miss1>=missmax) || (miss2>=missmax)) && (n<=nmax))
        eta_btmp2   = 0;
        eta_stmp2   = 0;
        
        for j=1:3
            nu_tmp      = poisson(eta_btmp1(n-1),eta_stmp1(n-1));
            R_tmp       = R(eta_btmp1(n-1),eta_stmp1(n-1));
            B_tmp       = B(j,0,eta_btmp1(n-1),nu_tmp,kappa,eta_stmp1(n-1));
            theta_tmp   = theta(j,eta_btmp1(n-1),eta_stmp1(n-1),nu_tmp,alpha1,kappa,g_tmp,phi_tmp,R_tmp);
            A_tmp       = A(j,alpha1,alpha3,eta_btmp1(n-1),g_tmp,R_tmp,1,phi_tmp,B_tmp,eta_stmp1(n-1),nu_tmp,kappa,0);
            eta_stmp2   = eta_stmp2+eta_srelaxed(A_tmp,c(j)*beta(i));
            eta_btmp2   = eta_btmp2+eta_brelaxed(theta_tmp,c(j)*beta(i));
        end
        eta_btmp1(n)    = 1/eta_btmp2;
        eta_stmp1(n)    = 1/(1/eta_stmp1(1)+eta_stmp2);
        
        %% ERROR FOR ETA_SR AND ETA_BR %%
        miss1   = abs(eta_btmp1(n)-eta_btmp1(n-1));
        miss2   = abs(eta_stmp1(n)-eta_stmp1(n-1));
        
%         disp(['iteration n=',num2str(n),', eta_b=',num2str(eta_btmp1(n)),', bulk error=',num2str(miss1),', eta_s=',num2str(num2str(eta_stmp1(n))),', shear error=',num2str(miss2)])
        
        n       = n+1;
    end
    %% ----------------------------- %%
%     disp(['ETA_B calculated for beta = ',num2str(beta(i)),' ! -> error @ iteration ',num2str(n-1),' = ',num2str(miss1)]);
%     disp(['ETA_S calculated for beta = ',num2str(beta(i)),' ! -> error @ iteration ',num2str(n-1),' = ',num2str(miss2)]);
    %% SET VALUES FOR BAR AND MY_RELAXED TERMS %%
    %% --------------------------------------- %%
    
    eta_s(i)        = eta_stmp1(n-1);
    eta_b(i)        = eta_btmp1(n-1);
    A_r(i)          = A_tmp;
    nu_r(i)         = poisson(eta_b(i),eta_s(i));
    %% --------------------------------------- %%
    
    waitbar(i/length(beta),wait,['beta=',sprintf('%1.6f',beta(i))]);
end
close(wait);

tablemodel      = datatable.getModel();
introw          = tablemodel.getRowCount();
tablemodel.setRowCount(introw+1);

datatable.setValueAt(modelname,introw,0);
datatable.setValueAt(betamin,introw,1);
datatable.setValueAt(betamax,introw,2);
datatable.setValueAt(eta_s0,introw,3);
datatable.setValueAt(alpha1,introw,4);
datatable.setValueAt(kappa,introw,5);
datatable.setValueAt(alpha3,introw,6);
datatable.setValueAt(c(1),introw,7);
datatable.setValueAt(c(2),introw,8);
datatable.setValueAt(c(3),introw,9);
datatable.setValueAt(nmax,introw,10);
datatable.setValueAt(unit,introw,11);

datatable.setModel(tablemodel);

mod_ind                  = tablemodel.getRowCount();

Vmodels(mod_ind).id      = tablemodel.getRowCount();
Vmodels(mod_ind).name    = modelname;
Vmodels(mod_ind).betamin = betamin;
Vmodels(mod_ind).betamax = betamax;
Vmodels(mod_ind).beta    = beta;
Vmodels(mod_ind).eta_s0  = eta_s0;
Vmodels(mod_ind).alpha1  = alpha1;
Vmodels(mod_ind).alpha3  = alpha3;
Vmodels(mod_ind).kappa   = kappa;
Vmodels(mod_ind).c1      = c(1);
Vmodels(mod_ind).c2      = c(2);
Vmodels(mod_ind).c3      = c(3);
Vmodels(mod_ind).nmax    = nmax;
Vmodels(mod_ind).eta_b   = eta_b;
Vmodels(mod_ind).eta_s   = eta_s;
Vmodels(mod_ind).nu_r    = nu_r;
Vmodels(mod_ind).A_r     = A_r;
Vmodels(mod_ind).unit    = unit;

namefield.setText(['model ',num2str(tablemodel.getRowCount()+1)]);
end